//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     cnst.cxx
// Purpose:    Ground term (constant)
// Last Change:10.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "atom.h"

#include "cnst.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor from Atom (Symbolic Constant):
//-----------------------------------------------------------------------------

cnst_c::cnst_c(atom_t atom)
{
	// Check input parameter:
	CHECK(atom != 0, "cnst_c::Constructor1: atom is NIL");
	CHECK(atom->valid(), "cnst_c::Constructor1: atom is invalid");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = CNST_MAGIC;
#endif

	// Initialize Member Variables:
	Kind = CNST_ATOM;
	Data.Atom = atom;
}

//-----------------------------------------------------------------------------
// Constructor from Integer:
//-----------------------------------------------------------------------------

cnst_c::cnst_c(int n)
{
	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = CNST_MAGIC;
#endif

	// Initialize Member Variables:
	Kind = CNST_INT;
	Data.Int = n;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

cnst_c::~cnst_c(void)
{
#ifdef VER_DEBUG
	Magic = 0;
#endif
}

//-----------------------------------------------------------------------------
// Debugging Support (Integrity Check):
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	cnst_c::valid(void) const
{
	// Check object pointer:
	CHECK(this != 0, "cnst_c::valid: NIL object");

	// The magic number must be right:
	if(Magic != CNST_MAGIC)
		return(BOOL_FALSE);

	// The term kind must be set:
	CHECK(Kind == CNST_ATOM || Kind == CNST_INT,
		"cnst_c::valid: Unknown kind of constant");

	// The atom cannot be NIL:
	CHECK(Kind != CNST_ATOM || Data.Atom != 0,
		"cnst_c::valid: Atom is NIL");

	// The atom must be valid:
	CHECK(Kind != CNST_ATOM || Data.Atom->valid(),
		"cnst_c::valid: Atom is invalid");

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// print: Print this term.
//-----------------------------------------------------------------------------

void cnst_c::print(void) const
{
	// Check consistency of this object:
	CHECK(valid(), "cnst_c::print: Bad object");

	// Print variable, atom, or integer:
	switch(Kind) {
		case CNST_ATOM:
			Data.Atom->print();
			break;
		case CNST_INT:
			out_c::print_int(Data.Int);
			break;
	}
}

